# -*- coding: utf-8 -*-

class Link(object):
    def __init__(self, node):
        self.node = node

    def text(self):
        return self.node.text

    def href(self):
        return self.node['href'] if self.node.has_attr('href') else ""

    def id(self):
        return self.node['id'] if self.node.has_attr('id') else ""



# -*- coding: utf-8 -*-

from .base_suite import BaseSuite
from motorise import Page, Form, Link
from bs4.element import Tag

class TestPage(BaseSuite):
    def test_title(self):
        page  = self.agent.get(self.url)
        title = page.title()

        self.assertEquals(title, "Google")

    def test_status_code(self):
        page = self.agent.get(self.url)
        code = page.status_code()

        self.assertEquals(code, 200)

    def test_html(self):
        page = self.agent.get(self.url)
        html = page.html()

        self.assertIsNotNone(html)

    def test_select(self):
        page   = self.agent.get(self.url)
        select = page.select("input[type=hidden]")[0]

        self.assertIsInstance(select, Tag)

    def test_form(self):
        page = self.agent.get(self.url)
        form = page.form("form[name=f]")

        self.assertIsInstance(form, Form)

    def test_submit(self):
        page   = self.agent.get(self.url)
        form   = page.form("form[name=f]")
        submit = form.submit()

        self.assertIsInstance(submit, Page)

    def test_links(self):
        page  = self.agent.get(self.url)
        links = page.links()
        link  = links[0]

        self.assertIsInstance(links, list)
        self.assertIsInstance(link, Link)

    def test_find_link(self):
        page = self.agent.get(self.url)

        name = page.find_link(name="YouTube")
        link = page.find_link(link="http://www.youtube.com/?gl=HK&tab=w1")

        name_regex = page.find_link(name_regex="Yo.*be")
        link_regex = page.find_link(link_regex=".*www.youtube.com.*")

        self.assertIsInstance(name, Link)
        self.assertIsInstance(link, Link)

        self.assertIsInstance(name_regex, Link)
        self.assertIsInstance(link_regex, Link)



# -*- coding: utf-8 -*-

from .base_suite import BaseSuite

class TestLink(BaseSuite):
    def test_text(self):
        page = self.agent.get(self.url)
        link = page.find_link(name="YouTube")

        self.assertEquals(link.text(), "YouTube")

    def test_href(self):
        page = self.agent.get(self.url)
        link = page.find_link(link="http://www.youtube.com/?gl=HK&tab=w1")

        self.assertEquals(link.href(), "http://www.youtube.com/?gl=HK&tab=w1")

    def test_id(self):
        page = self.agent.get(self.url)
        link = page.find_link(name="YouTube")

        self.assertEquals(link.id(), "")



# -*- coding: utf-8 -*-

import re
from bs4 import BeautifulSoup
from tidylib import tidy_document

from form import Form
from link import Link

class Page(object):
    def __init__(self, url, response, body, code, agent):
        self.url      = url
        self.response = response
        self.body     = body
        self.code     = code
        self.agent    = agent

        self.tidy_doc = tidy_document(body)[0]
        self.bs4_doc  = BeautifulSoup(self.tidy_doc)

        self.cached_links = []

    def title(self):
        title_node = self.bs4_doc.select("title")[0].string

        if not title_node:
            return ""
        else:
            return title_node.strip()

    def status_code(self):
        return self.code

    def html(self):
        return self.body

    def select(self, css_selector):
        return self.bs4_doc.select(css_selector)

    def form(self, css_selector):
        return Form(self, self.bs4_doc.select(css_selector)[0])

    def submit(self, form):
        return self.agent.submit(form)

    def links(self):
        if len(self.cached_links) == 0:
            for selector in ['a', 'area']:
                for link in self.bs4_doc.select(selector):
                    self.cached_links.append(Link(link))

        return self.cached_links

    def find_link(self, **kwargs):
        for link in self.links():
            link_name = link.text().strip()
            link_href = link.href().strip()

            if 'name' in kwargs and link_name == kwargs['name']:
                return link

            if 'link' in kwargs and link_href == kwargs['link']:
                return link

            if 'name_regex' in kwargs and re.match(kwargs['name_regex'], link_name):
                return link

            if 'link_regex' in kwargs and re.match(kwargs['link_regex'], link_href):
                return link

        return None



# -*- coding: utf-8 -*-

import re
import sys
import urlnorm

class Form(object):
    def __init__(self, page, form):
        self.page = page
        self.form = form

        self.action = self.form['action'] if self.form.has_attr('action') else page.url
        self.method = self.form['method'] if self.form.has_attr('method') else "get"
        self.name   = self.form['name']  if self.form.has_attr('name') else ""

        self.action = urlnorm.norm(self.page.url + self.action)

        self.fields = dict()
        self.load_hidden_fields()

    def load_hidden_fields(self):
        input_nodes = self.form.select("input[type=hidden]")

        for input_node in input_nodes:
            self.fields[input_node.get('name')] = input_node.get('value')

    def set_field(self, name, value):
        self.fields[name] = value

    def submit(self):
        return self.page.submit(self)



# -*- coding: utf-8 -*-

import unittest
from motorise import Agent

class BaseSuite(unittest.TestCase):
    def setUp(self):
        self.agent = Agent()
        self.url   = "http://www.google.com"



# -*- coding: utf-8 -*-

import requests

from page import Page

class Agent(object):
    def __init__(self):
        self.session = requests.Session()

    def get(self, url):
        response = self.session.get(url)

        return Page(url, response, response.text, response.status_code, self)

    def submit(self, form):
        if form.method.lower() == "get":
            response = self.session.get(form.action, params=form.fields)
        else:
            response = self.session.post(form.action, data=form.fields)

        return Page(form.action, response, response.text, response.status_code, self)



# -*- coding: utf-8 -*-

from .base_suite import BaseSuite
from motorise import Page

class TestAgent(BaseSuite):
    def test_get(self):
        page = self.agent.get(self.url)

        self.assertIsInstance(page, Page)

    def test_submit(self):
        page = self.agent.get(self.url)

        form   = page.form("form[name=f]")
        form.set_field("q", "github")
        submit = self.agent.submit(form)

        self.assertIsInstance(submit, Page)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from email.utils import parseaddr
from setuptools import setup, find_packages

__version__ = '0.1.3'
__author__ = 'Zeuxis Lo <seekstudio@gmail.com>'

author, author_email = parseaddr(__author__)

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name='motorise',
    version=__version__,
    url='https://github.com/zeuxisoo/python-motorise/',
    license='BSD',
    author=author,
    author_email=author_email,
    description='A toy for automating interaction with websites',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=requires,
)



# -*- coding: utf-8 -*-

from .base_suite import BaseSuite
from motorise import Form, Page

class TestForm(BaseSuite):
    def test_load_hidden_fields(self):
        page = self.agent.get(self.url)

        form = page.form("form[name=f]")

        self.assertIsNotNone(form.fields)

    def test_set_field(self):
        page = self.agent.get(self.url)

        form = page.form("form[name=f]")
        form.set_field('q', 'github')

        self.assertIn('q', form.fields)
        self.assertEquals('github', form.fields['q'])

    def test_submit(self):
        page = self.agent.get(self.url)

        form   = page.form("form[name=f]")
        form.set_field('q', 'github')
        submit = form.submit()

        self.assertIsInstance(submit, Page)
